#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from collections import namedtuple
from ansible.parsing.dataloader import DataLoader
from ansible.vars.manager import VariableManager
from ansible.inventory.manager import InventoryManager
from ansible.playbook.play import Play
from ansible.executor.task_queue_manager import TaskQueueManager
from ansible.plugins.callback import CallbackBase


class ResultCallback(CallbackBase):
    def v2_runner_on_ok(self, result, **kwargs):
        """ 每个 play.cmd打印一次 """
        host = result._host
        print json.dumps({host.name: result._result}, indent=4)

    def v2_runner_on_failed(self, result, **kwargs):
        host = result._host
        print json.dumps({host.name: result._result}, indent=4)

    def v2_runner_on_unreachable(self, result, **kwargs):
        host = result._host
        print json.dumps({host.name: result._result}, indent=4)


class PlayQueue(object):
    Options = namedtuple('Options', ['connection', 'module_path', 'forks', 'become', 'become_method', 'become_user', 'check', 'diff'])

    def __init__(self, module_path, user, sources):
        self.loader = DataLoader()
        self.options = self.Options(
            connection='smart',
            module_path=module_path, forks=100,
            become=None, become_method=None, become_user=user, check=False,
            diff=False
        )
        self.passwords = dict(vault_pass='secret')
        self.results_callback = ResultCallback()
        self.inventory = InventoryManager(loader=self.loader, sources=sources)
        self.variable_manager = VariableManager(loader=self.loader, inventory=self.inventory)
        self.tqm = TaskQueueManager(
            inventory=self.inventory,
            variable_manager=self.variable_manager,
            loader=self.loader,
            options=self.options,
            passwords=self.passwords,
            stdout_callback=self.results_callback,  # default is 'default'
        )
        self.plays = []

    def add_play(self, play):
        self.plays.append(
            Play().load(play.play_source, variable_manager=self.variable_manager, loader=self.loader)
        )

    def run(self):
        try:
            for play in self.plays:
                print '[ansible] %s流程执行结果: %s' % (play, self.tqm.run(play))
        finally:
            if self.tqm is not None:
                self.tqm.cleanup()
            self.plays = []


class EachPlay(object):
    def __init__(self, name, hosts):
        self.play_source = dict(
            name=name,
            hosts=hosts,
            gather_facts='no',
            tasks=[]
        )

    def add_cmd(self, module_name, module_args):
        self.play_source['tasks'].append(dict(action=dict(module=module_name, args=module_args), register='shell_out'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import platform
from setuptools import setup

setup(
    name='easy_ansible',
    version='1.0.1',
    author='CcccFz',
    author_email='ccccfz@163.com',
    url='https://github.com/CcccFz/easy-ansible',
    description='An ansible api that can be used directly and easily',
    packages=['easy_ansible'],
    install_requires=[],
    entry_points={}
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
easy_ansible library
~~~~~~~~~~~~~~~~~~~~~
An ansible api that can be used directly and easily.

:copyright: (c) 2017 by CcccFz.
:license: Apache2, see LICENSE for more details.

usage:
    See using example on 'test.py'.

"""

from ansible import __version__ as VERSION

if VERSION < '2':
    from ansible_19_api import Command
else:
    from ansible_24_api import PlayQueue, EachPlay



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ansible.runner import Runner
import json


class Command(object):
    def __init__(self, module_name='', module_args='', pattern=''):
        self.module_name = module_name
        self.module_args = module_args
        self.pattern = pattern
        self.results_raw = {}

    def run(self):
        runner = Runner(
            module_name=self.module_name,
            module_args=self.module_args,
            pattern=self.pattern,
            forks=10
        )
        self.results_raw = runner.run()
        print '[ansible] Ansible命令执行结果: \n%s' % json.dumps(self.results_raw, indent=4)

    @property
    def result(self):
        result = {}
        for k, v in self.results_raw.items():
            if k == 'dark':
                for host, info in v.items():
                    result[host] = {'dark': info.get('msg')}
            elif k == 'contacted':
                for host, info in v.items():
                    result[host] = {}
                    if info.get('stdout'):
                        result[host]['stdout'] = info.get('stdout')
                    elif info.get('stderr'):
                        result[host]['stderr'] = info.get('stderr')
        return result

    @property
    def state(self):
        result = {}
        if self.stdout:
            result['ok'] = self.stdout
        if self.stderr:
            result['err'] = self.stderr
        if self.dark:
            result['dark'] = self.dark

        return result

    @property
    def exec_time(self):
        """
        get the command execute time.
        """
        result = {}
        all = self.results_raw.get("contacted")
        for key, value in all.iteritems():
            result[key] = {
                "start": value.get("start"),
                "end": value.get("end"),
                "delta": value.get("delta"), }
        return result

    @property
    def stdout(self):
        """
        get the comamnd standard output.
        """
        result = {}
        all = self.results_raw.get("contacted")
        for key, value in all.iteritems():
            result[key] = value.get("stdout")
        return result

    @property
    def stderr(self):
        """
        get the command standard error.
        """
        result = {}
        all = self.results_raw.get("contacted")
        for key, value in all.iteritems():
            if value.get("stderr") or value.get("warnings"):
                result[key] = {
                    "stderr": value.get("stderr"),
                    "warnings": value.get("warnings"), }
        return result

    @property
    def dark(self):
        """
        get the dark results.
        """
        return self.results_raw.get("dark")